#    This source code file is part of the "mod_survey" package.
#
#    Copyright (C) 2004  Joel Palmius
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program (probably in a file named "LICENSE.txt" or the like);
#    if not, write to:
#
#    Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#!/usr/bin/perl

package Survey::Auth::DomainAuthEditor;
use strict;

use Survey::Language;

use CGI qw/:standard/;
use CGI::Cookie;

use constant MP2 => $mod_perl::VERSION < 1.99 ? 0 : 1;

use Safe;

sub checkLogin
{
    my ($login, $pass) = @_;

    if ((!$login) || (!$pass))
    {
        return 0;
    }

    my ($domainFile) = $ENV{"_SURVEY_SYSBASE"} . "domains/meta.domain";
    if (!-r $domainFile)
    {
        return 0;
    }

    my (%domainargs, $inlin);

    open(FIL, $domainFile) || return 0;
    while ($inlin = <FIL>)
    {
        chomp($inlin);
        if ($inlin)
        {
            my ($name, $arg) = split(/=/, $inlin, 2);
            $domainargs{$name} = $arg;

        }
    }
    close(FIL);

    if ($domainargs{"type"} eq "file")
    {
        if (&ScanFile($login, $pass, $domainargs{"filename"}))
        {
            return 1;
        }
    }

    return 0;
}

sub ScanFile
{
    my ($login, $pass, $file) = @_;

    my ($domainRoot) = $ENV{"_SURVEY_SYSBASE"} . "domains/";
    $file = $domainRoot . $file;

    if (!-r $file)
    {
        return 0;
    }

    open(FIN, $file) || return 0;

    while (<FIN>)
    {
        chomp $_;

        my ($userInFile, $passInFile) = split(/:/, $_, 2);
        my $salt = substr($passInFile, 0, 2);
        if ($login eq $userInFile)
        {
            if (crypt($pass, $salt) eq $passInFile)
            {
                close(FIN);
                return 1;
            }
        }
    }
    close(FIN);

    return 0;
}

sub readDomain
{
    my ($file) = shift;
    my (%args);

    my ($domainFile) = $ENV{"_SURVEY_SYSBASE"} . "domains/$file.domain";
    if (!-r $domainFile)
    {
        return \%args;
    }

    my ($inlin);

    open(FIL, $domainFile) || return 0;
    while ($inlin = <FIL>)
    {
        chomp($inlin);
        if ($inlin)
        {
            my ($name, $arg) = split(/=/, $inlin, 2);
            $args{$name} = $arg;
        }
    }
    close(FIL);

    return \%args;
}

sub mainMenu
{
    print "<html>\n";
    print "<head><title>Auth editor</title></head>\n";
    print "<body>\n";
    print "<h1>Auth editor</h1>\n";
    print "<h3>Domains</h3>\n";

    my ($domainRoot) = $ENV{"_SURVEY_SYSBASE"} . "domains/";
    open(PIPE, "ls $domainRoot/*.domain |") || die;
    my ($inlin);

    print "<table border=\"1\" cols=\"4\">\n";
    print "<tr><td>Domain</td><td>Type</td><td>Owner</td><td colspan=\"2\">Action</td></tr>\n";

    while ($inlin = <PIPE>)
    {
        my (@parts) = split(/\//, $inlin);
        $inlin = pop(@parts);
        my ($name, $crap) = split(/\./, $inlin, 2);

        my (%args) = %{ &readDomain($name) };

        print "<tr><td>$name</td>";
        print "<td>" . $args{"type"} . "</td>";
        print "<td>" . $args{"owner"} . "</td>";
        print "<td><a href=\"$name.edit?action=edit\">Edit</a></td>";
        print "<td>Delete</td>";
        print "</tr>\n";
    }
    print "</table>\n";

    close(PIPE);

    print "<br />\n";
    print "<h3>Add domain</h3>\n";
    print "<form action=\"?\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"action\" value=\"adddomain\" />\n";
    print "<input type=\"text\" name=\"name\" />\n";
    print "<input type=\"Submit\" value=\"Add\" />\n";
    print "</form>\n";

    print "</body>\n";
    print "</html>\n";
}

sub editMenu
{
    my ($domain) = shift;
    print "<html>\n";
    print "<head><title>Auth editor</title></head>\n";
    print "<body>\n";
    print "<h1>Auth editor</h1>\n";

    my (%args) = %{ &readDomain($domain) };
    print "Name:  " . $domain . "<br />";
    print "Type:  " . $args{"type"} . "<br />";
    print "File:  " . $args{"filename"} . "<br />";
    print "Owner: " . $args{"owner"} . "<br />";

    print "<h3>Users</h3>\n";

    my ($inlin);
    print "<form>\n";
    print "<select name=\"user\" size=\"10\" style=\"width: 300px\">\n";

    my ($filename) = $ENV{"_SURVEY_SYSBASE"} . "domains/" . $args{"filename"};

    open(FIL, "cat $filename | sort |") || die;

    while ($inlin = <FIL>)
    {
        chomp($inlin);
        my ($name, $pass) = split(/\:/, $inlin, 2);
        print "<option>$name</option>\n";
    }
    close(FIL);

    print "</select>\n";
    print "</form>\n";

    print "<br />\n";
    print "<h3>Add user</h3>\n";
    print "<form action=\"?\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"action\" value=\"adduser\" />\n";
    print "<tt>Name: </tt><input type=\"text\" name=\"name\" /><br />\n";
    print "<tt>Pass: </tt><input type=\"password\" name=\"pass\" /><br />\n";
    print "<input type=\"Submit\" value=\"Add\" />\n";
    print "</form>\n";

    print "</body>\n";
    print "</html>\n";
}

sub addDomainAction
{
    my ($login)    = shift;
    my ($name)     = param('name');
    my ($filename) = $ENV{"_SURVEY_SYSBASE"} . "domains/" . $name;
    if ((-e $filename . ".pwd") || (-e $filename . ".domain"))
    {
        return;
    }

    # More checks for name sanity here somewhere

    open(FIL, ">" . $filename . ".domain");
    print FIL "type=file\n";
    print FIL "filename=$name.pwd\n";
    print FIL "owner=$login\n";
    close(FIL);

    open(FIL, ">" . $filename . ".pwd");
    close(FIL);
}

sub addUserAction
{
    my ($domain)   = shift;
    my ($name)     = param('name');
    my ($pass)     = param('pass');
    my ($filename) = $ENV{"_SURVEY_SYSBASE"} . "domains/" . $domain;
    if ((!-e $filename . ".pwd") || (!-e $filename . ".domain"))
    {
        return;
    }

    my ($cpass) = crypt($pass, 'xx');

    open(FIL, ">>" . $filename . ".pwd");
    print FIL "$name:$cpass\n";
    close(FIL);
}

sub sanity
{
    my ($domain) = shift;

    my ($path) = $ENV{"_SURVEY_SYSBASE"} . "domains/";

    if (!-e $path)                       { return 0; }
    if (!-e $path . $domain . ".domain") { return 0; }
    if (!-e $path . $domain . ".pwd")    { return 0; }

    return 1;
}

sub handler
{
    my $r = shift;

    return DECLINED unless $r->content_type() eq 'text/html';

    if (MP2)
    {
        $r->headers_out->set('Content-type' => "text/html");
    }
    else
    {
        print "Content-type: text/html\n\n";
    }

    my ($login) = "";
    my ($pass)  = "";

    $pass  = ($r->get_basic_auth_pw())[1];
    $login = $r->user;

    my ($code) = &checkLogin($login, $pass);

    if ((!$code) || (!$login) || (!$pass))
    {
        $r->note_basic_auth_failure();
        return AUTH_REQUIRED;
    }

    my ($action) = param('action');

    my ($url)   = $r->uri;
    my (@parts) = split(/\//, $url);
    my ($uri)   = pop(@parts);
    my ($domain, $url) = split(/\./, $uri, 2);

    if (!&sanity($domain))
    {
        print "ERROR";
        return OK;
    }

    if ($action eq "adddomain")
    {
        &addDomainAction($login);
    }

    if ($action eq "adduser")
    {
        &addUserAction($domain);
    }

    if (($domain eq "meta") && ($action ne "edit"))
    {
        &mainMenu();
    }
    else
    {
        &editMenu($domain);
    }

    return OK;
}

1;

