#    This source code file is part of the "mod_survey" package.
#
#    Copyright (C) 2004  Joel Palmius
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program (probably in a file named "LICENSE.txt" or the like);
#    if not, write to:
#
#    Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#!/usr/bin/perl

package Survey::Auth::DomainAuthServer;
use strict;

use Survey::Language;

use CGI qw/:standard/;
use CGI::Cookie;

use constant MP2 => $mod_perl::VERSION < 1.99 ? 0 : 1;

use Safe;

sub ParseUri
{
    my ($r) = shift;

    my ($domain);
    my ($pass);
    my ($login);

    my ($url)   = $r->uri;
    my (@parts) = split(/\//, $url);
    my ($uri)   = pop(@parts);
    my ($domain, $url) = split(/\./, $uri, 2);

    $login = param('login') || "0";
    $pass  = param('pass')  || "0";

    return ($domain, $login, $pass);
}

sub checkLogin
{
    my ($domain, $login, $pass) = @_;

    if ((!$domain) || (!$login) || (!$pass))
    {
        return "ERROR";
    }

    my ($domainRoot) = $ENV{"_SURVEY_SYSBASE"} . "domains/";
    my ($domainFile) = $domainRoot . $domain . ".domain";
    if (!-r $domainFile)
    {
        return "ERROR";
    }

    my (%domainargs, $inlin);

    open(FIL, $domainFile) || return "ERROR";
    while ($inlin = <FIL>)
    {
        chomp($inlin);
        if ($inlin)
        {
            my ($name, $arg) = split(/=/, $inlin, 2);
            $domainargs{$name} = $arg;

        }
    }

    if ($domainargs{"type"} eq "file")
    {
        if (&ScanFile($login, $pass, $domainargs{"filename"}))
        {
            return "OK";
        }
    }

    return "DENIED";
}

sub ScanFile
{
    my ($login, $pass, $file) = @_;

    my ($domainRoot) = $ENV{"_SURVEY_SYSBASE"} . "domains/";
    $file = $domainRoot . $file;

    if (!-r $file)
    {
        return 0;
    }

    open(FIN, $file) || return 0;

    while (<FIN>)
    {
        chomp $_;

        my ($userInFile, $passInFile) = split(/:/, $_, 2);
        my $salt = substr($passInFile, 0, 2);
        if ($login eq $userInFile)
        {
            if (crypt($pass, $salt) eq $passInFile)
            {
                close(FIN);
                return 1;
            }
        }
    }
    close(FIN);

    return 0;
}

sub handler
{
    my $r = shift;

    return DECLINED unless $r->content_type() eq 'text/html';

    if (MP2)
    {
        $r->headers_out->set('Content-type' => "text/plain");
    }
    else
    {
        print "Content-type: text/plain\n\n";
    }

    my ($domain, $login, $pass) = &ParseUri($r);
    my ($code) = &checkLogin($domain, $login, $pass);

    print $code;

    return OK;
}

1;

