#    This source code file is part of the "mod_survey" package.
#
#    Copyright (C) 2004  Joel Palmius
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program (probably in a file named "LICENSE.txt" or the like);
#    if not, write to:
#
#    Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#!/usr/bin/perl

package Survey::Component::MailCopy;
use strict;

@Survey::Component::MailCopy::ISA = qw(Survey::Component::Component);

use Survey::Language;
use Text::ParseWords;
use Survey::Debug;

use CGI;
use Net::SMTP;

sub FillParams
{
    my ($crap, $self) = @_;

    @{ $self->{"ALLOWED_MAILCOPY"} } =
      ("FROM", "TO", "SUBJECT", "REPLYTO", "CAPTIONS", "SIGNATURE", "MSGAFTERDATA", "CC");
    @{ $self->{"ALL_MAILCOPY"} } = (@{ $self->{"ALLOWED_MAILCOPY"} }, "RAW", "TYPE");

    1;
}

sub FillDefaults
{
    my ($crap, $self) = @_;

    $self->{"MAILCOPY_TYPE"}         = "MAILCOPY";
    $self->{"MAILCOPY_FROM"}         = $ENV{"_SURVEY_MAIL_ADMIN"} || "root\@localhost";
    $self->{"MAILCOPY_TO"}           = $ENV{"_SURVEY_MAIL_ADMIN"} || "root\@localhost";
    $self->{"MAILCOPY_SUBJECT"}      = "[Mod_Survey] autogenerated message";
    $self->{"MAILCOPY_REPLYTO"}      = "";
    $self->{"MAILCOPY_SIGNATURE"}    = "";
    $self->{"MAILCOPY_MSGAFTERDATA"} = "no";
    $self->{"MAILCOPY_CAPTIONS"}     = "no";
    $self->{"MAILCOPY_CC"}           = "";

    1;
}

sub PlaceComponent
{
    my ($crap, $self, $paramstr) = @_;

    my ($tn) = "TAG" . $self->{NUMTAGS} . "_";

    my (%params, $cell, $name, $value, @pararr);

    %params = $self->GetDefaults("MAILCOPY");

    @pararr = @{$paramstr};

    foreach $cell (@pararr)
    {
        ($name, $value) = split(/=/, $cell);
        $params{$name} = $value;
        $self->CheckParam($name, "MAILCOPY");
    }

    if ($params{empty} eq "yes")
    {
        $self->{ERROR} =
          lprint("A MAILCOPY tag cannot be immediately terminated (in ") . $self->{FILE} . ")";
        $self->{ERRORCODE} = 10;
    }

    if (!$self->{ERROR})
    {
        my ($endtag) = index($self->{WORK}, "</MAILCOPY>");
        if ($endtag > -1)
        {
            $params{"RAW"} = substr($self->{WORK}, 0, $endtag - 1);
            $self->{WORK} =
              substr($self->{WORK}, 0 - length($self->{WORK}) + length($params{"RAW"}) + 12);
            $params{"RAW"} =~ s/^[\x0a\ ]+//;
            $params{"RAW"} =~ s/^[\x0a\x0d]+//;
            $params{"RAW"} =~ s/^\x0a+//;
            $params{"RAW"} =~ s/[\x0a\ ]+$//;
            $params{"RAW"} =~ s/[\x0a\x0d]+$//;
            $params{"RAW"} =~ s/\x0a+$//;
        }
        else
        {
            $self->{ERROR} =
              lprint("A MAILCOPY tag must have an end tag (in ") . $self->{FILE} . ")";
            $self->{ERRORCODE} = 14;
        }
    }

    $self->{MAILCOPYTAG} = $self->{NUMTAGS};

    $self->PlaceParams("MAILCOPY", $tn, %params);
    $self->{NUMTAGS}++;

    1;
}

sub NumberOfValues
{
    return 0;
}

sub GetValueNumerical
{
    return 0;
}

sub GetValueName
{
    return "";
}

sub PrintComponent
{
    my ($crap, $self, $tagno) = @_;
    my ($doc) = $self->{DOCUMENT};
    my ($out) = " ";

    return $out;
}

sub VariableParse
{

    #my($self,$value) = @_;
    my ($self, $doc, $sub, $arg, $ses, $value) = @_;

    if ($value =~ m/\{\$.*\$\}/)
    {
        my ($start) = index($value, "\{\$");
        my ($end)   = index($value, "\$\}");

        my ($varname) = substr($value, $start + 2, ($end - $start - 2));
        my ($varcont) = "[undefined]";

        if (defined($ses->getValue("SUBMITTED_$varname")))
        {
            $varcont = $ses->getValue("SUBMITTED_$varname");
        }

        $value =~ s/\{\$$varname\$\}/$varcont/;
    }

    return $value;
}

sub PerformMailing
{
    my ($self, $doc, $sub, $arg, $ses) = @_;

    my ($tagno) = $doc->{MAILCOPYTAG};

    if ($tagno >= 0)
    {
        my ($from)      = $doc->GetTagParam($tagno, "FROM");
        my ($to)        = $doc->GetTagParam($tagno, "TO");
        my ($subject)   = $doc->GetTagParam($tagno, "SUBJECT");
        my ($replyto)   = $doc->GetTagParam($tagno, "REPLYTO");
        my ($signature) = $doc->GetTagParam($tagno, "SIGNATURE");
        my ($after)     = $doc->GetTagParam($tagno, "MSGAFTERDATA");
        my ($cc)        = $doc->GetTagParam($tagno, "CC");
        my ($cap)       = $doc->GetTagParam($tagno, "CAPTIONS");
        my ($msg)       = $doc->GetTagParam($tagno, "RAW");

        $from    = Survey::Component::MailCopy->VariableParse($doc, $sub, $arg, $ses, $from);
        $to      = Survey::Component::MailCopy->VariableParse($doc, $sub, $arg, $ses, $to);
        $cc      = Survey::Component::MailCopy->VariableParse($doc, $sub, $arg, $ses, $cc);
        $replyto = Survey::Component::MailCopy->VariableParse($doc, $sub, $arg, $ses, $replyto);
        $subject = Survey::Component::MailCopy->VariableParse($doc, $sub, $arg, $ses, $subject);

        my ($smtphost) = $ENV{"_SURVEY_SMTP_HOST"} || "127.0.0.1";

        my ($smtp) = Net::SMTP->new($smtphost);

        if(!$smtp)
        {
          $sub->{ERRORCODE} = 99;
          $sub->{ERROR} = "MAILCOPY: Could not connect to $smtphost";
          return;
        }

        $smtp->mail($from);
        if ($cc)
        {
            $smtp->to($to, $cc);
        }
        else
        {
            $smtp->to($to);
        }

        $smtp->data();
        $smtp->datasend("X-Mailer: Mod_Survey 3.2.x\n");
        $smtp->datasend("User-Agent: Mod_Survey 3.2.x\n");
        $smtp->datasend("Content-Type: text/plain; charset=iso-8859-1\n");
        $smtp->datasend("From: $from\n");
        $smtp->datasend("To: $to\n");

        if ($replyto) { $smtp->datasend("Reply-To: $replyto\n"); }
        if ($cc)      { $smtp->datasend("CC: $cc\n"); }

        $smtp->datasend("Subject: $subject\n");

        $smtp->datasend("\n");

        if ($after eq "no")
        {
            $smtp->datasend("$msg\n\n");
        }

        my ($value);
        my (@data) = @{ $sub->{DATA} };

        if ($cap eq "no")
        {
            foreach $value (@data)
            {
                my ($name) = $value->{NAME};
                my ($val)  = $value->{VALUE};
                $smtp->datasend($name . ": " . $val . "\n");
            }
        }
        else
        {
            foreach $value (@data)
            {
                my ($name)  = $value->{NAME};
                my ($tn)    = $value->{TAGNO};
                my ($type)  = $value->{TYPE};
                my ($trans) = $doc->Translate($type);
                my ($val)   = $value->{VALUE};
                my ($valcap);

                my ($eval) =
                  "\$valcap = $trans->GetValueCaption(\$doc,\$sub,\$arg,\$ses,\$tn,\$val);";
                eval($eval);

                $smtp->datasend($name . ": " . $valcap . "\n");
            }
        }

        $smtp->datasend("\n");

        if ($after eq "yes")
        {
            $smtp->datasend("\n$msg\n\n");
        }

        if ($signature)
        {
            $smtp->datasend("$signature\n");
        }
        $smtp->datasend("\n\n[This message was automatically generated by Mod_Survey 3.2.x]\n\n");
        $smtp->dataend();

        $smtp->quit;

    }

    1;
}

1;

